SystemOrganization addCategory: #'Xpf-Server-Models'!
SystemOrganization addCategory: #'Xpf-Server-Tests'!
SystemOrganization addCategory: #'Xpf-Server-Rest'!
SystemOrganization addCategory: #'Xpf-Server-Rest-Tests'!

RestHttpHandler subclass: #XpfRestBackupHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestBackupHttpHandler methodsFor: 'testing' stamp: 'SR 3/17/2009 16:42'!
isAuthorized: anHeaderField
	| user |
	
	user := XpfUser authenticate: anHeaderField userid by: anHeaderField password.
	^ user notNil and: [user isAdmin]
	! !

!XpfRestBackupHttpHandler methodsFor: 'authentication' stamp: 'SR 3/17/2009 16:41'!
needsAuthentication
	^ true! !

!XpfRestBackupHttpHandler methodsFor: 'request processing' stamp: 'mig 3/25/2009 16:17'!
process: anHTTPRequest
	"Handles an http request. Should answer with an http response."
	
	^ (super process: anHTTPRequest) ifNil: [
		HTTPResponse ok entity: {XpfProject all. XpfUser all. XpfUser adminUser. XpfEntity lastId} sixxString.
	]
! !

RestHttpHandler subclass: #XpfRestFeedbackHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestFeedbackHttpHandler methodsFor: 'authentication' stamp: 'SR 4/15/2009 14:23'!
needsAuthentication

	^ false! !

!XpfRestFeedbackHttpHandler methodsFor: 'request processing' stamp: 'SR 4/15/2009 14:38'!
process: anHTTPRequest
	"Handles an http request. Should answer with an http response."
	
	^ HTTPResponse ok entity:''! !

RestHttpHandler subclass: #XpfRestInfoHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestInfoHttpHandler methodsFor: 'testing' stamp: 'SR 3/9/2009 15:42'!
isAuthorized: anHeaderField

	^ (XpfUser authenticate: anHeaderField userid by: anHeaderField password) notNil
	! !

!XpfRestInfoHttpHandler methodsFor: 'authentication' stamp: 'mig 4/1/2009 15:19'!
needsAuthentication

	^ true! !

!XpfRestInfoHttpHandler methodsFor: 'request processing' stamp: 'SR 3/9/2009 16:04'!
process: anHTTPRequest
	"Handles an http request. Should answer with an http response."
	
	^ (super process: anHTTPRequest) ifNil: [ | user |
		user := XpfUser findByLogin: (anHTTPRequest headers fieldNamed: 'AUTHORIZATION') userid.
		
		HTTPResponse ok entity: (
			(XpfRestResourceHttpHandler uriMapping at: user class) value: user).
	]
! !

!Object methodsFor: '*xpf-server-models' stamp: 'mfp 3/6/2009 11:45'!
isXpfEntity
	^ false! !

Object subclass: #XpfEntity
	instanceVariableNames: 'id revision isDeleted'
	classVariableNames: 'LastId'
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfEntity commentStamp: 'mt 2/12/2009 12:20' prior: 0!
I am the abstract superclass of all entities in the system. These entities could be user stories, tasks, developers, themes, ...!

!XpfEntity class methodsFor: 'description' stamp: 'mt 4/3/2009 11:54'!
collectionPropertyNames

	^ #()! !

!XpfEntity class methodsFor: 'private' stamp: 'topa 4/7/2009 14:03'!
incrementId

	self lastId: self lastId + 1.
	^ self lastId! !

!XpfEntity class methodsFor: 'accessing' stamp: 'mt 4/16/2009 15:27'!
lastId

	^ LastId ifNil: [self lastId: 1. self lastId]! !

!XpfEntity class methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:01'!
lastId: anObject 

	LastId := anObject.! !

!XpfEntity class methodsFor: 'description' stamp: 'mt 4/21/2009 14:17'!
primitivePropertyNames

	^ #(id revision isDeleted)! !

!XpfEntity class methodsFor: 'description' stamp: 'topa 3/3/2009 22:15'!
referencePropertyNames

	^ #()! !

!XpfEntity methodsFor: 'as yet unclassified' stamp: 'test 4/21/2009 14:25'!
delete

	self pvtIsDeleted: true.! !

!XpfEntity methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:13'!
id

	^ id! !

!XpfEntity methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:08'!
id: anObject

	id := anObject.! !

!XpfEntity methodsFor: 'revisions' stamp: 'topa 4/2/2009 18:42'!
incrementRevision

	self pvtRevision: self revision + 1.! !

!XpfEntity methodsFor: 'as yet unclassified' stamp: 'mt 4/21/2009 13:58'!
isDeleted

	^ self pvtIsDeleted! !

!XpfEntity methodsFor: 'as yet unclassified' stamp: 'mt 4/21/2009 14:17'!
isDeleted: aBoolean
	"Read-only."
	
	^ self! !

!XpfEntity methodsFor: 'authorization' stamp: 'topa 4/7/2009 14:08'!
isMethod: aMethodString allowedFor: aUser
	"returns whether the method is allowed for the specified user; may be overwritten in subclasses"

	^ true! !

!XpfEntity methodsFor: 'testing' stamp: 'topa 3/24/2009 17:27'!
isXpfEntity

	^ true! !

!XpfEntity methodsFor: 'printing' stamp: 'SR 4/16/2009 23:44'!
jsonWriteOn: aStream
	"Write its own properties onto the stream:
	'{'title'->'Just an entity', 
	  'ref project' -> '/projects/3', 
	  'col userStories' -> ['/..', '/..']}'
	'ref' indicates a reference to another resource
	'col' indicates a collection of other resources"

	| result |
	self flag: #NeedsRefactoring.
	result := Dictionary newFrom: (self class primitivePropertyNames collect: [ :aSymbol |
			aSymbol -> (self perform: aSymbol)]).
	result addAll: (Dictionary newFrom: (self class referencePropertyNames collect: [ :aSymbol | 
			'ref ', aSymbol -> ((self perform: aSymbol)
				ifNil: [nil]
				ifNotNil: [(self perform: aSymbol) restUri])])).
	result addAll: (Dictionary newFrom: (self class collectionPropertyNames collect: [ :aSymbol |
			'col ', aSymbol -> ((self perform: aSymbol) collect: [:each | each restUri])])).
	result addAll: (Dictionary newFrom: (self class collectionPropertyNames collect: [:aSymbol | 
			'col ', aSymbol, 'WithDeleted' -> ((self perform: (aSymbol, 'WithDeleted') asSymbol) collect: [:each | each restUri])])).
	result jsonWriteOn: aStream.
	! !

!XpfEntity methodsFor: 'as yet unclassified' stamp: 'mt 4/21/2009 13:55'!
pvtIsDeleted

	^ isDeleted ifNil: [false]! !

!XpfEntity methodsFor: 'as yet unclassified' stamp: 'mt 4/21/2009 13:55'!
pvtIsDeleted: aBoolean

	isDeleted := aBoolean.! !

!XpfEntity methodsFor: 'private' stamp: 'topa 4/2/2009 18:42'!
pvtRevision: aNumber

	revision := aNumber.! !

!XpfEntity methodsFor: 'printing' stamp: 'mt 2/25/2009 10:12'!
restUri

	^ (XpfRestResourceHttpHandler uriMapping at: self class) value: self! !

!XpfEntity methodsFor: 'accessing' stamp: 'topa 4/2/2009 18:42'!
revision

	^ revision ifNil: [revision := 1]! !

!XpfEntity methodsFor: 'accessing' stamp: 'mt 4/6/2009 16:51'!
revision: anObject
	"rest and GemStone compatibility:"

	^ self! !

XpfEntity subclass: #XpfEntityWithHistory
	instanceVariableNames: 'history'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfEntityWithHistory class methodsFor: 'description' stamp: 'cs 4/17/2009 11:56'!
collectionPropertyNames

	^ super collectionPropertyNames, #(history)! !

!XpfEntityWithHistory class methodsFor: 'description' stamp: 'cs 4/17/2009 11:48'!
historyActionNames

	^ #(created changed deleted added removed)! !

!XpfEntityWithHistory methodsFor: 'as yet unclassified' stamp: 'cs 4/17/2009 11:40'!
delete

	super delete.
	self historyAddEntry: #deleted.! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 11:36'!
history

	history ifNil: [history := OrderedCollection new].
	^ history! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 11:37'!
history: anObject
	"History is read-only."

	^ self
	! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 11:37'!
historyAddEntry: action

	self historyAddEntry: action for: nil.! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 11:37'!
historyAddEntry: action for: property

	self historyAddEntry: action for: property to: nil.! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 12:02'!
historyAddEntry: action for: property from: oldValue to: newValue
	"Add entry to entity's history."
	
	self history add:
		(XpfHistoryEntry
			entity: self
			action: action
			property: property
			from: oldValue
			to: newValue).
	self incrementRevision.! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 11:37'!
historyAddEntry: action for: property to: newValue

	self historyAddEntry: action for: property from: nil to: newValue.! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 12:08'!
historyWithDeleted

	^ self history! !

!XpfEntityWithHistory methodsFor: 'as yet unclassified' stamp: 'cs 4/17/2009 11:40'!
initialize

	super initialize.
	self historyAddEntry: #created.! !

XpfEntityWithHistory subclass: #XpfIteration
	instanceVariableNames: 'project title userStories timeIdeal'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

XpfIteration subclass: #XpfFutureIteration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfFutureIteration methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:18'!
id

	^ -1! !

!XpfFutureIteration methodsFor: 'testing' stamp: 'topa 3/24/2009 17:18'!
isXpfFutureIteration

	^ true! !

!XpfFutureIteration methodsFor: 'as yet unclassified' stamp: 'topa 4/7/2009 14:12'!
moveNilUserStoriesToFuture
	
	self project userStories do: [:us | 
		us iteration ifNil: [self addUserStory: us]].! !

!XpfFutureIteration methodsFor: 'as yet unclassified' stamp: 'SR 4/12/2009 21:15'!
timeIdeal

	^ 0! !

!XpfFutureIteration methodsFor: 'as yet unclassified' stamp: 'SR 4/12/2009 21:15'!
timeIdeal: anObject

	^ self! !

!XpfFutureIteration methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:19'!
title

	^ 'Future'! !

!XpfFutureIteration methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:14'!
title: aString
	"rest and GemStone compatibility"
	
	^ self! !

!XpfIteration class methodsFor: 'as yet unclassified' stamp: 'SR 4/7/2009 11:33'!
collectionPropertyNames

	^ super collectionPropertyNames, #(userStories)! !

!XpfIteration class methodsFor: 'as yet unclassified' stamp: 'SR 3/25/2009 18:13'!
newFrom: aDictionary

	^ self new title: (aDictionary at: 'title' ifAbsent: [])! !

!XpfIteration class methodsFor: 'as yet unclassified' stamp: 'SR 4/12/2009 21:15'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title isXpfFutureIteration timeIdeal)! !

!XpfIteration class methodsFor: 'as yet unclassified' stamp: 'SR 4/7/2009 11:33'!
referencePropertyNames

	^ super referencePropertyNames, #(project)! !

!XpfIteration methodsFor: 'add-remove' stamp: 'SR 3/25/2009 18:15'!
addNewUserStory: aDictionary

	^ self addUserStory: (XpfUserStory newFrom: aDictionary)! !

!XpfIteration methodsFor: 'add-remove' stamp: 'mt 4/16/2009 12:59'!
addUserStory: aUserStory 

	aUserStory id ifNil: [
		aUserStory id: aUserStory class incrementId].

	aUserStory iteration ifNotNil: [
			aUserStory iteration detachUserStory: aUserStory].

	aUserStory iteration: self.
	(self pvtUserStories includes: aUserStory) ifFalse: [
		self pvtUserStories add: aUserStory.
		self historyAddEntry: #added for: #userStories to: aUserStory].
	
	^ aUserStory! !

!XpfIteration methodsFor: 'add-remove' stamp: 'mt 4/16/2009 12:55'!
delete

	self userStories do: [:us | us delete].
	self project historyAddEntry: #removed for: #iterations to: self.
	super delete.
	! !

!XpfIteration methodsFor: 'private' stamp: 'mt 4/16/2009 13:04'!
detachUserStory: aUserStory

	(self userStories noneSatisfy: [:us | us = aUserStory])
		ifTrue: [^ aUserStory].
	self pvtUserStories remove: aUserStory.
	self historyAddEntry: #removed for: #userStories to: aUserStory.	
	aUserStory iteration: nil.
	^ aUserStory! !

!XpfIteration methodsFor: 'initialization' stamp: 'mt 4/16/2009 11:09'!
initialize

	super initialize.
	self pvtUserStories: OrderedCollection new.! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 4/7/2009 11:15'!
isXpfFutureIteration

	^ false! !

!XpfIteration methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:11'!
isXpfFutureIteration: aBoolean
	"rest and GemStone compatibility"
	
	^ self! !

!XpfIteration methodsFor: 'printing' stamp: 'SR 3/20/2009 20:56'!
printString

	^ self title ifNil: ['No Title']! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 4/7/2009 11:15'!
project

	^ project! !

!XpfIteration methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:11'!
project: anObject

	self incrementRevision.
	project := anObject.! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:59'!
pvtUserStories

	^ userStories! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:59'!
pvtUserStories: anObject

	userStories := anObject.! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 4/12/2009 21:15'!
timeIdeal

	^ timeIdeal! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 4/12/2009 21:15'!
timeIdeal: anObject

	timeIdeal := anObject! !

!XpfIteration methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:21'!
title

	^ title! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:26'!
title: anObject

	anObject = self title ifTrue: [^ self].
	self historyAddEntry: #changed for: #title from: self title to: anObject.
	title := anObject.! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:59'!
userStories

	^ userStories reject: [:us | us isDeleted]! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/16/2009 15:35'!
userStoriesWithDeleted

	^ userStories copy! !

XpfEntityWithHistory subclass: #XpfProject
	instanceVariableNames: 'title developers iterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

XpfProject class
	instanceVariableNames: 'projects testProjects useTestData'!

XpfProject class
	instanceVariableNames: 'projects testProjects useTestData'!

!XpfProject class methodsFor: 'add-remove' stamp: 'SR 3/25/2009 18:10'!
addNewProject: aDictionary

	^ self addProject: (self newFrom: aDictionary)! !

!XpfProject class methodsFor: 'add-remove' stamp: 'mt 4/6/2009 14:25'!
addProject: aProject

	(self pvtProjects includes: aProject) ifTrue: [^ aProject].
	aProject id: self incrementId.
	^ self pvtProjects add: aProject! !

!XpfProject class methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:24'!
all

	^ self projects! !

!XpfProject class methodsFor: 'accessing' stamp: 'SR 4/17/2009 08:40'!
allWithDeleted

	^ self projectsWithDeleted! !

!XpfProject class methodsFor: 'description' stamp: 'mt 4/16/2009 13:31'!
collectionPropertyNames

	^ super collectionPropertyNames, #(developers iterations)! !

!XpfProject class methodsFor: 'initialization' stamp: 'mig 4/17/2009 19:30'!
initialize

	super initialize.
	self
		useTestData: false;
		projects: Set new;
		testProjects: Set new.! !

!XpfProject class methodsFor: 'instance creation' stamp: 'SR 3/25/2009 18:08'!
newFrom: aDictionary

	^ self new title: (aDictionary at: 'title')! !

!XpfProject class methodsFor: 'description' stamp: 'topa 3/3/2009 22:26'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title)! !

!XpfProject class methodsFor: 'accessing' stamp: 'mt 4/6/2009 15:36'!
projects

	^ (self useTestData
		ifTrue: [self testProjects]
		ifFalse: [self pvtProjects]) reject: [:project | project isDeleted]! !

!XpfProject class methodsFor: 'accessing' stamp: 'mt 4/6/2009 15:30'!
projects: aCollection

	self useTestData
		ifTrue: [self testProjects: aCollection]
		ifFalse: [projects := aCollection]! !

!XpfProject class methodsFor: 'accessing' stamp: 'SR 4/17/2009 08:40'!
projectsWithDeleted

	^ (self useTestData
		ifTrue: [self testProjects]
		ifFalse: [self pvtProjects])! !

!XpfProject class methodsFor: 'accessing' stamp: 'mt 4/6/2009 11:55'!
pvtProjects

	^ self useTestData
		ifTrue: [self testProjects]
		ifFalse: [projects]! !

!XpfProject class methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:46'!
testProjects

	^ testProjects! !

!XpfProject class methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:04'!
testProjects: anObject

	testProjects := anObject.! !

!XpfProject class methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:25'!
useTestData

	^ useTestData! !

!XpfProject class methodsFor: 'testing' stamp: 'topa 4/7/2009 14:04'!
useTestData: aBoolean

	useTestData := aBoolean.! !

!XpfProject methodsFor: 'add-remove' stamp: 'mt 4/16/2009 13:03'!
addIteration: anIteration 

	(anIteration isXpfFutureIteration or: [self pvtIterations includes: anIteration]) 
		ifTrue: [^ anIteration].
	self pvtIterations add: anIteration before: (self pvtIterations last).
	anIteration
		project: self;
		id: anIteration class incrementId.
	self historyAddEntry: #added for: #iterations to: anIteration.
	^ anIteration! !

!XpfProject methodsFor: 'add-remove' stamp: 'SR 3/25/2009 18:13'!
addNewIteration: aDictionary

	^ self addIteration: (XpfIteration newFrom: aDictionary)! !

!XpfProject methodsFor: 'add-remove' stamp: 'mt 4/16/2009 13:11'!
attachDeveloper: aUser

	self pvtDevelopers add: aUser ifPresent: [^ aUser].
	aUser attachProject: self.
	self historyAddEntry: #added for: #developers to: aUser.	
	^ aUser! !

!XpfProject methodsFor: 'deleting' stamp: 'mt 4/16/2009 11:36'!
delete
	
	self iterations do: [:iter | iter delete].
	super delete.! !

!XpfProject methodsFor: 'add-remove' stamp: 'mt 4/16/2009 13:18'!
detachDeveloper: aUser

	self pvtDevelopers remove: aUser ifAbsent: [^ aUser].
	self historyAddEntry: #removed for: #developers to: aUser.
	aUser detachProject: self.
	^ aUser! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:41'!
developers

	^ developers reject: [:dev | dev isDeleted]! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/16/2009 15:36'!
developersWithDeleted

	^ developers copy! !

!XpfProject methodsFor: 'initialization' stamp: 'mt 4/16/2009 11:08'!
initialize

	super initialize.
	self
		pvtIterations: (OrderedCollection with: (XpfFutureIteration new project: self));
		pvtDevelopers: OrderedCollection new.
! !

!XpfProject methodsFor: 'authorization' stamp: 'cs 3/18/2009 15:28'!
isMethod: aMethodString allowedFor: aUser

	^ self developers includes: aUser! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/6/2009 15:35'!
iterations

	^ self pvtIterations reject: [:it | it isDeleted]! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/16/2009 15:38'!
iterationsWithDeleted

	^ iterations copy! !

!XpfProject methodsFor: 'printing' stamp: 'mt 2/25/2009 10:16'!
printString

	^ self title ifNil: ['No Title']! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:41'!
pvtDevelopers

	^ developers! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:41'!
pvtDevelopers: anObject

	developers := anObject.! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/6/2009 11:52'!
pvtIterations

	^ iterations! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/6/2009 15:23'!
pvtIterations: anObject

	iterations := anObject.! !

!XpfProject methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:24'!
title

	^ title! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:26'!
title: anObject

	anObject = self title ifTrue: [^ self].
	self historyAddEntry: #changed for: #title from: self title to: anObject.
	title := anObject.! !

XpfEntityWithHistory subclass: #XpfTask
	instanceVariableNames: 'title userStory timeEstimated timeActual developers isDone responsibleDeveloper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfTask commentStamp: 'mt 2/12/2009 12:22' prior: 0!
I am a task in tems of Extreme Programming.!

!XpfTask class methodsFor: 'description' stamp: 'mt 4/15/2009 17:10'!
collectionPropertyNames

	^ super collectionPropertyNames, #(developers)! !

!XpfTask class methodsFor: 'instance creation' stamp: 'mt 4/15/2009 16:58'!
newFrom: aDictionary

	^ self new
		title: (aDictionary at: 'title' ifAbsent: []);
		timeActual: (aDictionary at: 'timeActual' ifAbsent: []);
		timeEstimated: (aDictionary at: 'timeEstimated' ifAbsent: []);
		isDone: (aDictionary at: 'isDone' ifAbsent: [false])! !

!XpfTask class methodsFor: 'description' stamp: 'mt 4/16/2009 13:31'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title timeEstimated timeActual isDone)! !

!XpfTask class methodsFor: 'description' stamp: 'mt 4/16/2009 13:31'!
referencePropertyNames

	^ super referencePropertyNames, #(responsibleDeveloper userStory)! !

!XpfTask methodsFor: 'add-remove' stamp: 'mt 4/16/2009 13:13'!
attachDeveloper: aUser

	self pvtDevelopers add: aUser ifPresent: [^ aUser].
	aUser attachTask: self.
	self historyAddEntry: #added for: #developers to: aUser.
	^ aUser! !

!XpfTask methodsFor: 'deleting' stamp: 'mt 4/16/2009 12:56'!
delete

	self userStory historyAddEntry: #removed for: #tasks to: self.
	super delete.! !

!XpfTask methodsFor: 'add-remove' stamp: 'mt 4/16/2009 13:19'!
detachDeveloper: aUser

	self pvtDevelopers remove: aUser ifAbsent: [^ aUser].
	self historyAddEntry: #removed for: #developers to: aUser.
	aUser detachTask: self.
	^ aUser! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:19'!
developers

	^ developers reject: [:dev | dev isDeleted]! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 15:39'!
developersWithDeleted

	^ developers copy! !

!XpfTask methodsFor: 'initialization' stamp: 'mt 4/16/2009 11:19'!
initialize

	super initialize.
	self pvtDevelopers: OrderedCollection new.! !

!XpfTask methodsFor: 'accessing' stamp: 'test 4/16/2009 13:36'!
isDone

	^ isDone ifNil: [isDone := false]! !

!XpfTask methodsFor: 'accessing' stamp: 'test 4/16/2009 13:35'!
isDone: anObject

	anObject = self isDone ifTrue: [^ self].
	self historyAddEntry: #changed for: #isDone from: self isDone to: anObject.
	isDone := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:18'!
pvtDevelopers

	^ developers! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:18'!
pvtDevelopers: anObject

	developers := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'SR 4/10/2009 15:07'!
responsibleDeveloper

	^ responsibleDeveloper! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:08'!
responsibleDeveloper: anObject

	responsibleDeveloper := anObject.
	self attachDeveloper: anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
timeActual

	^ timeActual! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:26'!
timeActual: anObject

	anObject = self timeActual ifTrue: [^ self].
	self historyAddEntry: #changed for: #timeActual from: self timeActual to: anObject.
	timeActual := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
timeEstimated

	^ timeEstimated! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:27'!
timeEstimated: anObject

	anObject = self timeEstimated ifTrue: [^ self].
	self historyAddEntry: #changed for: #timeEstimated from: self timeEstimated to: anObject.
	timeEstimated := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
title

	^ title! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:27'!
title: anObject

	anObject = self title ifTrue: [^ self].
	self historyAddEntry: #changed for: #title from: self title to: anObject.
	title := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/15/2009 16:59'!
userStory

	^ userStory! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:27'!
userStory: anXpfUserStory

	self userStory = anXpfUserStory ifTrue: [^ anXpfUserStory].
	userStory := anXpfUserStory.
	self historyAddEntry: #changed for: #userStory from: self userStory to: anXpfUserStory.! !

XpfEntityWithHistory subclass: #XpfUser
	instanceVariableNames: 'userName email projects password tasks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

XpfUser class
	instanceVariableNames: 'users adminUser testUsers useTestData currentUser'!

!XpfUser commentStamp: 'mt 2/12/2009 12:24' prior: 0!
I am a system or application user. This could be an administrator, webmaster, developer or something like that.!

XpfUser class
	instanceVariableNames: 'users adminUser testUsers useTestData currentUser'!

!XpfUser class methodsFor: 'add-remove' stamp: 'SR 3/25/2009 18:02'!
addNewUser: aDictionary

	^ self addUser: (self newFrom: aDictionary)! !

!XpfUser class methodsFor: 'add-remove' stamp: 'mt 4/6/2009 14:28'!
addUser: aUser

	(self pvtUsers includes: aUser) ifTrue: [^ aUser].
	aUser id: self incrementId.
	^ self pvtUsers add: aUser! !

!XpfUser class methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:47'!
adminUser

	^ adminUser! !

!XpfUser class methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:05'!
adminUser: anObject

	adminUser := anObject.! !

!XpfUser class methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:37'!
all

	^ self users! !

!XpfUser class methodsFor: 'accessing' stamp: 'SR 4/17/2009 08:39'!
allWithDeleted

	^ self usersWithDeleted! !

!XpfUser class methodsFor: 'authentication' stamp: 'mt 4/3/2009 11:02'!
authenticate: aLogin by: aPassword
	
	^ self all 
		detect: [ :aUser| (aUser login = aLogin) and: [aUser authenticateBy: aPassword]]
		ifNone: []! !

!XpfUser class methodsFor: 'description' stamp: 'topa 3/3/2009 22:37'!
collectionPropertyNames

	^ super collectionPropertyNames, #(tasks projects)! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 4/3/2009 11:06'!
currentUser

	^ currentUser! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 4/3/2009 11:06'!
currentUser: anXpfUser

	currentUser := anXpfUser.! !

!XpfUser class methodsFor: 'authentication' stamp: 'SR 3/9/2009 15:46'!
findByLogin: aLoginString

	^ self all detect: [:user | user login = aLoginString]! !

!XpfUser class methodsFor: 'authentication' stamp: 'topa 3/3/2009 22:37'!
hash: aString

	^ SecureHashAlgorithm new hashMessage: aString! !

!XpfUser class methodsFor: 'initialization' stamp: 'mig 4/17/2009 19:30'!
initialize

	super initialize.
	self 
		useTestData: false;
		users: Set new;
		adminUser: (self addNewUser: (Dictionary newFrom: {
			'userName' -> 'Administrator'. 
			'login' -> 'admin@example.com'. 
			'password' -> 'nimda'}));
		testUsers: Set new.! !

!XpfUser class methodsFor: 'instance creation' stamp: 'SR 3/25/2009 18:01'!
newFrom: aDictionary

	^ self new
		userName: (aDictionary at: 'userName' ifAbsent: []);
		login: (aDictionary at: 'login');
		password: (aDictionary at: 'password' ifAbsent: [])! !

!XpfUser class methodsFor: 'description' stamp: 'mt 3/6/2009 13:56'!
primitivePropertyNames

	^ super primitivePropertyNames, #(login userName email password)! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 4/6/2009 11:56'!
pvtUsers 

	^ self useTestData
		ifTrue: [self testUsers]
		ifFalse: [users]! !

!XpfUser class methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:48'!
testUsers

	^ testUsers! !

!XpfUser class methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:06'!
testUsers: anObject

	testUsers := anObject.! !

!XpfUser class methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:48'!
useTestData

	^ useTestData! !

!XpfUser class methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:06'!
useTestData: anObject

	useTestData := anObject.! !

!XpfUser class methodsFor: 'accessing' stamp: 'mt 4/6/2009 15:36'!
users 

	^ (self useTestData
		ifTrue: [self testUsers]
		ifFalse: [self pvtUsers]) reject: [:user | user isDeleted]! !

!XpfUser class methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:06'!
users: aCollection

	self useTestData
		ifTrue: [self testUsers: aCollection]
		ifFalse: [users := aCollection].! !

!XpfUser class methodsFor: 'accessing' stamp: 'SR 4/17/2009 08:39'!
usersWithDeleted 

	^ (self useTestData
		ifTrue: [self testUsers]
		ifFalse: [self pvtUsers])! !

!XpfUser methodsFor: 'add-remove' stamp: 'mt 4/16/2009 13:11'!
attachProject: aProject

	self pvtProjects add: aProject ifPresent: [^ aProject].
	aProject attachDeveloper: self.
	self historyAddEntry: #added for: #projects to: aProject.
	^ aProject! !

!XpfUser methodsFor: 'add-remove' stamp: 'mt 4/16/2009 13:12'!
attachTask: aTask 

	self pvtTasks add: aTask ifPresent: [^ aTask].
	aTask attachDeveloper: self.
	self historyAddEntry: #added for: #tasks to: aTask.
	^ aTask
! !

!XpfUser methodsFor: 'authentication' stamp: 'SR 4/10/2009 12:18'!
authenticateBy: aString

	aString ifEmpty: [^ false]. "empty password is evil"
	^ self pvtPassword = (self class hash: aString)! !

!XpfUser methodsFor: 'deleting' stamp: 'mt 4/16/2009 13:16'!
delete

	self isAdmin ifTrue: [^ self].
	
	self projects do: [:project | 
		project historyAddEntry: #removed for: #developers to: self].
	self tasks do: [:task |
		task historyAddEntry: #removed for: #developers to: self].
	super delete.! !

!XpfUser methodsFor: 'add-remove' stamp: 'mt 4/16/2009 13:17'!
detachProject: aProject 

	self pvtProjects remove: aProject ifAbsent: [^ aProject].
	self historyAddEntry: #removed for: #projects to: aProject.
	aProject detachDeveloper: self.
	^ aProject! !

!XpfUser methodsFor: 'add-remove' stamp: 'mt 4/16/2009 13:19'!
detachTask: aTask

	self pvtTasks remove: aTask ifAbsent: [^ aTask].
	self historyAddEntry: #removed for: #tasks to: aTask.
	aTask detachDeveloper: self.
	^ aTask! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:33'!
email

	^ email! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:27'!
email: anObject

	anObject = self email ifTrue: [^ self].
	self historyAddEntry: #changed for: #email from: self email to: anObject.
	email := anObject.! !

!XpfUser methodsFor: 'initialization' stamp: 'mt 4/16/2009 11:25'!
initialize

	super initialize.
	self 
		pvtProjects: OrderedCollection new;
		pvtTasks: OrderedCollection new.! !

!XpfUser methodsFor: 'authentication' stamp: 'mt 3/6/2009 13:08'!
isAdmin

	^ self = self class adminUser! !

!XpfUser methodsFor: 'as yet unclassified' stamp: 'mig 4/15/2009 15:50'!
isMethod: aMethodString allowedFor: aUser

	^ (aMethodString = 'PUT')
		ifTrue: [(self = aUser) or: [aUser isAdmin]]
		ifFalse: [true]
! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 3/4/2009 14:05'!
login

	^ email! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:27'!
login: anObject
	"You have to choose a valid email address as a login name."

	anObject = self email ifTrue: [^ self].
	self historyAddEntry: #changed for: #email from: self email to: anObject.
	email := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:21'!
password
	"The password is never actually returned but compared to.
	This method is necessary for rest, too."
	
	^ nil! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:01'!
password: aString

	(aString notNil and: [(self class hash: aString) = password]) ifTrue: [^ self].
	self historyAddEntry: #changed for: #password.
	password := aString ifNotNil: [self class hash: aString].! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/15/2009 17:39'!
projects

	^ projects reject: [:project | project isDeleted]! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 15:40'!
projectsWithDeleted

	^ projects copy! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:35'!
pvtPassword

	^ password! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/15/2009 17:49'!
pvtProjects

	^ projects! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:25'!
pvtProjects: anObject

	projects := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:24'!
pvtTasks

	^ tasks! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:24'!
pvtTasks: anObject

	tasks := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/15/2009 17:39'!
tasks

	^ tasks reject: [:task | task isDeleted]! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 15:40'!
tasksWithDeleted

	^ tasks copy! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 3/4/2009 13:54'!
userName

	^ userName! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:28'!
userName: anObject

	self userName = anObject ifTrue: [^ self].
	self historyAddEntry: #changed for: #username from: self userName to: anObject.
	userName := anObject.! !

XpfEntityWithHistory subclass: #XpfUserStory
	instanceVariableNames: 'title priority timeEstimated tasks timeActual iteration positionX positionY color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfUserStory commentStamp: 'topa 2/12/2009 14:06' prior: 0!
I am a user story in terms of Extreme Programming!

!XpfUserStory class methodsFor: 'description' stamp: 'topa 3/3/2009 22:40'!
collectionPropertyNames

	^ super collectionPropertyNames, #(tasks)! !

!XpfUserStory class methodsFor: 'instance creation' stamp: 'mt 4/15/2009 16:57'!
newFrom: aDictionary

	^ self new
		title: (aDictionary at: 'title' ifAbsent: []);
		priority: (aDictionary at: 'priority' ifAbsent: []);
		timeEstimated: (aDictionary at: 'timeEstimated' ifAbsent: []);
		timeActual: (aDictionary at: 'timeActual' ifAbsent: [])! !

!XpfUserStory class methodsFor: 'description' stamp: 'mt 4/16/2009 13:32'!
primitivePropertyNames

	^ super primitivePropertyNames, 
		#(title priority timeActual timeEstimated positionX positionY color)! !

!XpfUserStory class methodsFor: 'description' stamp: 'mt 4/16/2009 13:32'!
referencePropertyNames

	^ super referencePropertyNames, #(iteration)! !

!XpfUserStory methodsFor: 'add-remove' stamp: 'SR 3/25/2009 18:00'!
addNewTask: aPropertyDictionary

	^ self addTask: (XpfTask newFrom: aPropertyDictionary)! !

!XpfUserStory methodsFor: 'add-remove' stamp: 'mt 4/16/2009 13:00'!
addTask: aTask

	aTask id ifNil: [
		aTask id: aTask class incrementId].
	
	aTask userStory ifNotNil: [
		aTask userStory detachTask: aTask].
	
	aTask userStory: self.
	(self pvtTasks includes: aTask) ifFalse: [
		self pvtTasks add: aTask.
		self historyAddEntry: #added for: #tasks to: aTask].
	
	^ aTask! !

!XpfUserStory methodsFor: 'accessing' stamp: 'SR 3/18/2009 15:25'!
color

	^ color! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:28'!
color: anObject

	anObject = self color ifTrue: [^ self].
	self historyAddEntry: #changed for: #color from: self color to: anObject.
	color := anObject.! !

!XpfUserStory methodsFor: 'deleting' stamp: 'mt 4/16/2009 12:56'!
delete

	self tasks do: [:task | task delete].
	self iteration historyAddEntry: #removed for: #userStories to: self.
	super delete.! !

!XpfUserStory methodsFor: 'private' stamp: 'mt 4/16/2009 13:00'!
detachTask: aTask

	(self tasks noneSatisfy: [:t | t = aTask])
		ifTrue: [^ aTask].
	self pvtTasks remove: aTask.
	self historyAddEntry: #removed for: #tasks to: aTask.
	aTask userStory: nil.
	^ aTask! !

!XpfUserStory methodsFor: 'initialization' stamp: 'mt 4/15/2009 16:42'!
initialize

	super initialize.
	self 
		pvtTasks: OrderedCollection new;
		positionX: 0.0;
		positionY: 0.0;
		color: 'white'.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:40'!
iteration

	^ iteration! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:29'!
iteration: anObject

	self iteration = anObject ifTrue: [^ self].
	self historyAddEntry: #changed for: #iteration from: self iteration to: anObject.
	iteration := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:41'!
positionX

	^ positionX! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/6/2009 17:03'!
positionX: anObject

	anObject = positionX ifTrue: [^ self].
	self incrementRevision.
	positionX := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:41'!
positionY

	^ positionY! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/6/2009 17:04'!
positionY: anObject

	anObject = positionY ifTrue: [^ self].
	self incrementRevision.
	positionY := anObject.! !

!XpfUserStory methodsFor: 'printing' stamp: 'mt 2/25/2009 10:21'!
printString

	^ self title ifNil: ['No Title']! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:39'!
priority

	^ priority! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:29'!
priority: anObject

	anObject = self priority ifTrue: [^ self].
	self historyAddEntry: #changed for: #priority from: self priority to: anObject.
	priority := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/15/2009 16:41'!
pvtTasks

	^ tasks! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/15/2009 16:53'!
pvtTasks: anObject

	tasks := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/15/2009 16:41'!
tasks

	^ self pvtTasks reject: [:task | task isDeleted]! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 15:40'!
tasksWithDeleted

	^ tasks copy! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
timeActual

	^ timeActual! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:29'!
timeActual: anObject

	anObject = self timeActual ifTrue: [^ self].
	self historyAddEntry: #changed for: #timeActual from: self timeActual to: anObject.
	timeActual := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
timeEstimated

	^ timeEstimated! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:29'!
timeEstimated: anObject

	anObject = self timeEstimated ifTrue: [^ self].
	self historyAddEntry: #changed for: #timeEstimated from: self timeEstimated to: anObject.
	timeEstimated := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
title

	^ title! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:29'!
title: anObject

	anObject = self title ifTrue: [^ self].
	self historyAddEntry: #changed for: #title from: self title to: anObject.
	title := anObject.! !

XpfEntity subclass: #XpfHistoryEntry
	instanceVariableNames: 'entity timestamp user action property oldVal newVal oldRef newRef'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfHistoryEntry class methodsFor: 'as yet unclassified' stamp: 'cs 4/17/2009 12:04'!
entity: entity action: action property: property from: from to: to

	^ self new
		entity: entity;
		action: action;
		property: property;
		old: from;
		new: to! !

!XpfHistoryEntry class methodsFor: 'description' stamp: 'cs 4/17/2009 11:57'!
primitivePropertyNames

	^ super primitivePropertyNames, #(timestamp action property oldVal newVal)! !

!XpfHistoryEntry class methodsFor: 'description' stamp: 'cs 4/17/2009 12:03'!
referencePropertyNames

	^ super referencePropertyNames, #(entity user oldRef newRef)! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:44'!
action

	^ action! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:44'!
action: anObject

	action := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 12:01'!
entity

	^ entity! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 12:01'!
entity: anObject

	entity := anObject.! !

!XpfHistoryEntry methodsFor: 'as yet unclassified' stamp: 'cs 4/17/2009 13:03'!
initialize

	super initialize.
	self
		id: self class incrementId;
		timestamp: TimeStamp now printString;
		user: XpfUser currentUser.! !

!XpfHistoryEntry methodsFor: 'as yet unclassified' stamp: 'cs 4/17/2009 13:04'!
isMethod: aMethodString allowedFor: aUser
	"returns whether the method is allowed for the specified user; may be overwritten in subclasses"

	^ self entity isMethod: aMethodString allowedFor: aUser! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:52'!
new: anObject

	anObject isXpfEntity
		ifTrue: [self newRef: anObject]
		ifFalse: [self newVal: anObject].! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:44'!
newRef

	^ newRef.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:44'!
newRef: anObject

	newRef := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
newVal

	^ newVal! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
newVal: anObject

	newVal := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:51'!
old: anObject

	anObject isXpfEntity
		ifTrue: [self oldRef: anObject]
		ifFalse: [self oldVal: anObject].! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
oldRef

	^ oldRef! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
oldRef: anObject

	oldRef := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
oldVal

	^ oldVal! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
oldVal: anObject

	oldVal := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
property

	^ property! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
property: anObject

	property := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
timestamp

	^ timestamp! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
timestamp: anObject

	timestamp := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:46'!
user

	^ user! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:46'!
user: anObject

	user := anObject.! !

Object subclass: #XpfEntityRevisionWrapper
	instanceVariableNames: 'clientRevision entity restUri isValid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:17'!
clientRevision

	^ clientRevision! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:17'!
clientRevision: anObject

	clientRevision := anObject.! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:17'!
entity

	^ entity! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:17'!
entity: anObject

	entity := anObject.! !

!XpfEntityRevisionWrapper methodsFor: 'as yet unclassified' stamp: 'mt 4/21/2009 13:18'!
initialize

	super initialize.
	self isValid: true.! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mt 4/21/2009 13:15'!
isValid

	^ isValid! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mt 4/21/2009 13:15'!
isValid: anObject 

	isValid := anObject.! !

!XpfEntityRevisionWrapper methodsFor: 'testing' stamp: 'mt 4/21/2009 15:35'!
needsUpdate

	^ self entity isNil
		or: [self isValid not
		or: [self entity isDeleted
		or: [self entity revision > self clientRevision]]]! !

!XpfEntityRevisionWrapper methodsFor: 'resolving' stamp: 'mt 4/21/2009 15:30'!
resolve: anHTTPRequest handler: handler

	[self entity:
		(self resolveUri: self restUri request: anHTTPRequest handler: handler)]
		on: RestError
		do: [:e | self isValid: false. ^ self].
	self entity isCollection ifTrue:
		[self entity:
			(self resolveUri: self shortRestUri request: anHTTPRequest handler: handler)]! !

!XpfEntityRevisionWrapper methodsFor: 'request processing' stamp: 'mig 4/1/2009 18:08'!
resolveUri: aString request: anHTTPRequest handler: handler

	^ aString isEmpty
		ifTrue: []
		ifFalse: [RestRequestProcessor new
			handler: handler;
			request: anHTTPRequest;
			resolveUri: aString]
! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:17'!
restUri

	^ restUri! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:18'!
restUri: anObject

	restUri := anObject.! !

!XpfEntityRevisionWrapper methodsFor: 'resolving' stamp: 'mig 4/1/2009 17:55'!
shortRestUri

	| result |
	result := String new.
	(self restUri findTokens: $/) allButLast
		do: [:each | result := result, each]
		separatedBy: [result := result, '/'].
	^ result! !

Object subclass: #XpfService
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfService class methodsFor: 'as yet unclassified' stamp: 'SR 4/15/2009 14:24'!
start
	RestHttpService start.
	RestHttpService resource
		installHandler: XpfRestBackupHttpHandler on: '/backup';
		installHandler: XpfRestFeedbackHttpHandler on: '/feedback';
		installHandler: XpfRestInfoHttpHandler on: '/info';
		installHandler: XpfRestRevisionHttpHandler on: '/'.! !

!XpfService class methodsFor: 'as yet unclassified' stamp: 'SR 3/9/2009 16:20'!
stop
	RestHttpService stop.! !

RestResourceHttpHandler subclass: #XpfRestResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestResourceHttpHandler class methodsFor: 'constants' stamp: 'cs 4/17/2009 12:00'!
rootResources

	^ Dictionary newFrom: {
		'projects' -> XpfProject.
		'users' -> XpfUser}! !

!XpfRestResourceHttpHandler class methodsFor: 'constants' stamp: 'cs 4/17/2009 12:01'!
uriMapping

	^ Dictionary newFrom:{
		XpfUser -> [:user| '/users/{1}' format: {user id}].		
		XpfProject -> [:project| '/projects/{1}' format: {project id}].
		XpfIteration -> [:iter | '{1}/iterations/{2}' format: {iter project restUri. iter id}].
		XpfFutureIteration -> [:iter | '{1}/iterations/{2}' format: {iter project restUri. iter id}].			
		XpfUserStory -> [:us | '{1}/userStories/{2}' format: {us iteration restUri. us id}].
		XpfTask -> [:task | '{1}/tasks/{2}' format: {task userStory restUri. task id}].		
		XpfHistoryEntry -> [:hist | '{1}/history/{2}' format: {hist entity restUri. hist id}].}
! !

!XpfRestResourceHttpHandler methodsFor: 'as yet unclassified' stamp: 'mt 4/3/2009 10:58'!
getUserOf: aRequest

	| headerField |	
	headerField := aRequest headers 
		fieldNamed: 'AUTHORIZATION' 
		ifNone: [RestNotAllowedError signal: 'No authorization information supplied'].
		
	^ XpfUser authenticate: headerField userid by: headerField password.! !

!XpfRestResourceHttpHandler methodsFor: 'testing' stamp: 'topa 3/5/2009 14:30'!
isAuthorized: anHeaderField

	^ (XpfUser authenticate: anHeaderField userid by: anHeaderField password) notNil
	! !

!XpfRestResourceHttpHandler methodsFor: 'authentication' stamp: 'topa 3/5/2009 13:45'!
needsAuthentication

	^ true! !

!XpfRestResourceHttpHandler methodsFor: 'as yet unclassified' stamp: 'SR 4/21/2009 13:41'!
process: anHTTPRequest
	"Save current user context for writing into the history of the entity. Clear that context after processing the request."

	| response |
	response := super process: anHTTPRequest.
	^ response! !

!XpfRestResourceHttpHandler methodsFor: 'authentication' stamp: 'SR 4/21/2009 13:41'!
requestAllowed: aRequest for: anObject

	| user |
	user := self getUserOf: aRequest.
	XpfUser currentUser: user.
	user isAdmin ifTrue: [^ self].

	(anObject isXpfEntity and: [(anObject isMethod: aRequest class methodName allowedFor: user) not])
		ifTrue: [RestNotAllowedError signal: 'Access forbidden for this resource'].! !

!XpfRestResourceHttpHandler methodsFor: 'authentication' stamp: 'SR 4/16/2009 22:56'!
selectorAllowed: aSelector for: anObject

	^ (super selectorAllowed: aSelector for: anObject) and:
		[(((anObject class collectionPropertyNames union: 
		(anObject class collectionPropertyNames collect: [:name | (name asString, 'WithDeleted') asSymbol])) union:
		anObject class primitivePropertyNames) union: 
		(anObject class referencePropertyNames) ) 
		includes: aSelector]
	  ! !

XpfRestResourceHttpHandler subclass: #XpfRestRevisionHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestRevisionHttpHandler methodsFor: 'request processing' stamp: 'mt 4/21/2009 13:11'!
findChanges: anHTTPRequest

	| wrapper result |
	result := OrderedCollection new.
	(self postDataFor: anHTTPRequest) keysAndValuesDo:	[ :restUri :revision | 
		wrapper := XpfEntityRevisionWrapper new
			restUri: restUri;
			clientRevision: revision;
			resolve: anHTTPRequest handler: self.
			wrapper needsUpdate ifTrue: [ result add: wrapper restUri ]].
	^ self responseFor: result! !

!XpfRestRevisionHttpHandler methodsFor: 'request processing' stamp: 'mig 4/1/2009 15:53'!
postDataFor: anHTTPRequest

	 | data |
	anHTTPRequest class category = 'Hyper' ifFalse: [anHTTPRequest ensureFullRead].
	data := Json readFrom: anHTTPRequest entityBody value readStream.
	^ data! !

!XpfRestRevisionHttpHandler methodsFor: 'request processing' stamp: 'mig 4/1/2009 15:31'!
process: anHTTPRequest
	"Handles an http request. Should answer with an http response."
	
	^ (anHTTPRequest uri identifierPath size = 2 and:
		[anHTTPRequest uri identifierPath second = 'changes'])
			ifTrue: [self findChanges: anHTTPRequest]
			ifFalse: [super process: anHTTPRequest]! !

!XpfRestRevisionHttpHandler methodsFor: 'request processing' stamp: 'mig 4/1/2009 15:46'!
responseFor: aCollection

	^ HTTPResponse ok entity: (Json render: aCollection)! !

RestTest subclass: #XpfRestTest
	instanceVariableNames: 'testProject testUser testUser2 testTask projects users testStory testIteration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest-Tests'!

XpfRestTest subclass: #XpfRestAuthTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest-Tests'!

!XpfRestAuthTest methodsFor: 'as yet unclassified' stamp: 'mt 4/21/2009 14:55'!
testAuthentication

	"Try to get information about a project via http GET request.
	 Verify this by looking into the database directly."

	| response jsonResult |
	self should: XpfUser currentUser isNil.
	response := self get: 'projects/', testProject id asString request: self class auth.
	self
		should: response responseCode = 401;
		should: XpfUser currentUser isNil.
	
	response := self get: 'projects/', testProject id asString request: self class authUser2Pass.
	self should: response responseCode = 403.

	response := self get: 'projects/', testProject id asString request: self class authUserPass.
	self should: response responseCode = 200.

	jsonResult := Json readFrom: response content readStream.
	
 	self 
		should: testProject title = (jsonResult at: 'title');
		should: testProject id = (jsonResult at: 'id').! !

XpfRestTest subclass: #XpfRestResourceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest-Tests'!

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:42'!
testDeleteOneProject
	"Try to delete a project via http DELETE request. This operation should be idempotent. 
	Verify this by looking into the database directly."

	| projectCount response |
	projectCount := XpfProject all size.
	response := self delete: testProject restUri.
	
	self 
		should: response responseCode = 200;
		should: XpfProject all size = (projectCount - 1);
		shouldnt: (XpfProject all anySatisfy: [:aProject| aProject id = testProject id]).! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:41'!
testDeleteOneTask
	"Try to delete a task from testProject via http DELETE request.
	 This operation should be idempotent. Verify this by looking into the database directly."

	| taskCount response |
	taskCount := testStory tasks size.
	response := self delete: testTask restUri.
	
	self
		should: response responseCode = 200;
		should: testStory tasks size = (taskCount - 1);
		shouldnt: (testStory tasks anySatisfy: [:task| task id = testTask id]).! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:42'!
testDeleteOneUser
	"Try to delete a user via http DELETE request. This operation should be idempotent. 
	Verify this by looking into the database directly."

	| userCount response |
	userCount := XpfUser all size.
	response := self delete: testUser restUri.
	
	self 
		should: response responseCode = 200;
		should: XpfUser all size = (userCount - 1);
		shouldnt: (XpfUser all anySatisfy: [:aUser| aUser id = testUser id]).! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:44'!
testDeleteUserstoryWithThreeTasks
	"Try to delete a userStory from testProject via http DELETE request."

	| response storyCount |
	testStory addNewTask: (Dictionary newFrom: {'title' -> 'Test Task 2'}).
	testStory addNewTask: (Dictionary newFrom: {'title' -> 'Test Task 3'}).
	storyCount := testIteration userStories size.
	response := self delete: testStory restUri.
	self
		should: response responseCode = 200;
		should: testIteration userStories size = (storyCount - 1);
		should: testStory tasks size = 0.! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:44'!
testGetAllProjects
	"Try to get a list with rest urls to all projects via http GET request. 
	Verify this by looking into the database directly."

	| response jsonResult restUrl |
	response := self get: 'projects'.	
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	
	self should: XpfProject all size = jsonResult size.
	
	XpfProject all do: [:project |
		restUrl := project restUri.
		self should: (jsonResult includes: restUrl)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:46'!
testGetAllTasks
	"Try to get a list with rest urls to all tasks of testProject via http GET request.
	 Verify this by looking into the database directly."

	| response restUrl jsonResult|
	response := self get: ('{1}/tasks' format: {testStory restUri}).
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	self should: testStory tasks size = jsonResult size.
	
	testStory tasks do: [:task |
		restUrl := task restUri.
		self should: (jsonResult includes: restUrl)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'SR 4/16/2009 23:34'!
testGetAllTasksWithDeleted
	"Try to get a list with rest urls to all tasks of testProject via http GET request.
	 Verify this by looking into the database directly."

	| response restUrl jsonResult|
	testTask delete.
	response := self get: ('{1}/tasks/all' format: {testStory restUri}).
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	self
		should: testStory tasks size = (testStory tasksWithDeleted size - 1);
		should: testStory tasksWithDeleted size = jsonResult size.
	
	testStory tasksWithDeleted do: [:task |
		restUrl := task restUri.
		self should: (jsonResult includes: restUrl)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:47'!
testGetAllUsers
	"Try to get a list with rest urls to all users via http GET request. 
	Verify this by looking into the database directly."

	| response jsonResult restUrl |
	response := self get: 'users'.	
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	
	self should: XpfUser all size = jsonResult size.
	
	XpfUser all do: [:user |
		restUrl := user restUri.
		self should: (jsonResult includes: restUrl)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 15:20'!
testGetOneDeletedTask
	"Try to get information about a task from testProject via http GET request.
	 Verify this by looking into the database directly."

	| response jsonResult |
	testTask delete.
	response := self get: testTask restUri.
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	self
		should: testTask title = (jsonResult at: 'title');
		should: testTask id = (jsonResult at: 'id');
		should: testTask timeEstimated = (jsonResult at: 'timeEstimated');
		should: testTask timeActual = (jsonResult at: 'timeActual');
		should: testTask isDone = (jsonResult at: 'isDone');
		should: testTask userStory restUri = (jsonResult at: 'ref userStory');
		should: testTask developers size = (jsonResult at: 'col developers') size.	
	
	testTask developers do: [:developer |
		self should: ((jsonResult at: 'col developer') includes: developer restUri)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:50'!
testGetOneProject
	"Try to get information about a project via http GET request.
	 Verify this by looking into the database directly."

	| response jsonResult |	
	response := self get: 'projects/', testProject id asString.
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	
	self 
		should: testProject title = (jsonResult at: 'title');
		should: testProject id = (jsonResult at: 'id');
		should: testProject iterations size = (jsonResult at: 'col iterations') size;
		should: testProject developers size = (jsonResult at: 'col developers') size.	
	
	testProject iterations do: [:iter | 
		self should: ((jsonResult at: 'col iterations') includes: iter restUri)].
	
	testProject developers do: [:developer |
		self should: ((jsonResult at: 'col developers') includes: developer restUri)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:52'!
testGetOneTask
	"Try to get information about a task from testProject via http GET request.
	 Verify this by looking into the database directly."

	| response jsonResult |
	response := self get: testTask restUri.
	self should: response responseCode = 200.
	
	jsonResult := Json readFrom: response content readStream.
	self
		should: testTask title = (jsonResult at: 'title');
		should: testTask id = (jsonResult at: 'id');
		should: testTask timeEstimated = (jsonResult at: 'timeEstimated');
		should: testTask timeActual = (jsonResult at: 'timeActual');
		should: testTask isDone = (jsonResult at: 'isDone');
		should: testTask userStory restUri = (jsonResult at: 'ref userStory');
		should: testTask developers size = (jsonResult at: 'col developers') size.	
	
	testTask developers do: [:developer |
		self should: ((jsonResult at: 'col developer') includes: developer restUri)].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:52'!
testPostAddProject
	"Try to create a new project via http POST request. Verify this by looking into the database directly."

	| response projectCount newProject |
	projectCount := XpfProject all size.
	response := self post: 'projects' content: (self jsonDataFrom: {'title' -> 'new test project'}).

	self should: XpfProject all size = (projectCount + 1).

	newProject := XpfProject all detect: [:aProject | 
		aProject id = response content asURI pathComponents last asNumber].
	self 
		should: newProject title = 'new test project';
		should: response content = newProject restUri! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 14:55'!
testPostAddTask
	"Try to create a new task to testProject via http POST request.
	 Verify this by looking into the database directly."
	
	| response tasksCount newTask |
	self flag: #CreateFixtures.
	tasksCount := testStory tasks size.
	response := self
		post: ('{1}/tasks' format: {testStory restUri})
		content: (self jsonDataFrom: {'title' -> 'new test task'. 'timeEstimated' -> 42}).
	
	self should: response responseCode = 200.
	
	response := response content.
	
	self should: testStory tasks size = (tasksCount + 1).
	
	newTask := testStory tasks detect:
		[:task | task id = response asURI pathComponents last asNumber].
	
	self
		should: response = newTask restUri;
		should: newTask title = 'new test task';
		should: newTask timeEstimated = 42;
		should: newTask userStory = testStory.! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:55'!
testPostAddUser
	"Try to create a new user via http POST request. Verify this by looking into the database directly."

	| response userCount newUser |
	userCount := XpfUser all size.
	response := self
		post: 'users'
		content: (self jsonDataFrom: {'userName' -> 'New User'. 'login' -> 'new.user@email.de'. 'password' -> nil}).

	self should: XpfUser all size = (userCount + 1).

	newUser := XpfUser all detect: [:aProject | 
		aProject id = response content asURI pathComponents last asNumber].
	self 
		should: newUser userName = 'New User';
		should: newUser login = 'new.user@email.de';
		should: response content = newUser restUri.! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:57'!
testPostAttachDeveloperToProject
	"Try to attach testUser to testProject via http POST request"
	
	| response developersCount |
	developersCount := testProject developers size.
	response := self
		post: ('{1}/developers' format: {testProject restUri})
		content: ('"{1}"' format: {testUser2 restUri}).
	
	self 
		should: response responseCode = 200;
		should: testProject developers size = (developersCount + 1);
		should: (testProject developers includes: testUser2);
		should: response content = testUser2 restUri.! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:58'!
testPostAttachDeveloperToTask
	"Try to attach testUser to testTask via http POST request.
	 Verify this by looking into the database directly."
	
	| response developersCount |
	developersCount := testTask developers size.
	response := self
		post: ('{1}/developers' format: {testTask restUri})
		content: ('"{1}"' format: {testUser restUri}).
	
	self 
		should: response responseCode = 200;
		should: testTask developers size = (developersCount + 1);
		should: (testTask developers includes: testUser);
		should: response content = testUser restUri.! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 14:00'!
testPutOneProject
	"Try to update information for a project via http PUT request.
	 This operation should be idempotent. Verify this by looking into the database directly."
	
	| projectCount response iterCount developersCount |
	projectCount := XpfProject all size.
	iterCount := testProject iterations size.
	developersCount := testProject developers size.
	
	2 timesRepeat: [	
		response := self
			put: testProject restUri
			content: (self jsonDataFrom: {'title' -> 'New Title'}).
			
		self 
			should: response responseCode = 200;
			should: XpfProject all size = projectCount;
			should: testProject title = 'New Title';
			should: testProject iterations size = iterCount;
			should: testProject developers size = developersCount].! !

!XpfRestResourceTest methodsFor: 'tests' stamp: 'mt 4/16/2009 14:02'!
testPutOneTask
	"Try to update information for a task of testProject via http PUT request.
	 This operation should be idempotent. Verify this by looking into the database directly."
	
	| taskCount response developersCount |
	self flag: #CreateFixtures.
	taskCount := testStory tasks size.
	developersCount := testTask developers size.
	
	2 timesRepeat: [
		response := self
			put: testTask restUri
			content: (self jsonDataFrom: {
				'title' -> 'New Title'.
				'timeEstimated' -> 25.
				'timeActual' -> 50.
				'isDone' -> true}).
			
		self 
			should: response responseCode = 200;
			should: testStory tasks size = taskCount;
			should: testTask title = 'New Title';
			should: testTask timeEstimated = 25;
			should: testTask timeActual = 50;
			should: testTask userStory = testStory;
			should: testTask developers size = developersCount;
			should: testTask isDone].! !

!XpfRestTest class methodsFor: 'testing' stamp: 'topa 3/5/2009 13:08'!
isAbstract
	
	^ self == XpfRestTest
! !

!XpfRestTest methodsFor: 'actions' stamp: 'mt 3/12/2009 13:10'!
backup

	XpfProject useTestData: true.
	XpfUser useTestData: true.
	
	XpfProject projects: Set new.
	XpfUser users: Set new.! !

!XpfRestTest methodsFor: 'actions' stamp: 'mt 3/12/2009 13:10'!
restore

	XpfProject useTestData: false.
	XpfUser useTestData: false.! !

!XpfRestTest methodsFor: 'running' stamp: 'mt 4/16/2009 13:39'!
setUp
	"Install a handler for http service. Add a test project to the database."

	super setUp.
	self backup.
	RestHttpService resource installHandler: XpfRestRevisionHttpHandler on: '/'.

	testUser := XpfUser new login: self class login; password: self class password. 
	testUser2 := XpfUser new login: self class login2; password: self class password.

	testProject := XpfProject new title: 'Test Project'.
	testIteration := XpfIteration new title: 'Test Iteration'.
	testStory := XpfUserStory new title: 'Test UserStory'.
	testTask := XpfTask new
				title: 'Test Task';
				timeEstimated: 12;
				timeActual: 15.
	XpfProject addProject: testProject.
	XpfUser addUser: testUser.
	XpfUser addUser: testUser2.
	testProject addIteration: testIteration.
	testIteration addUserStory: testStory.
	testStory addTask: testTask.
	testProject attachDeveloper: testUser.! !

!XpfRestTest methodsFor: 'running' stamp: 'mt 4/3/2009 11:10'!
tearDown

	self restore.
	super tearDown.
	
	XpfUser currentUser: nil.
! !

XpfRestTest subclass: #XpfRevisionHandlerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest-Tests'!

!XpfRevisionHandlerTest methodsFor: 'as yet unclassified' stamp: 'mig 4/1/2009 16:02'!
testChanges

	| response changes localRevisions |
	localRevisions := Dictionary new
		at: testUser restUri put: testUser revision;
		at: testUser2 restUri put: testUser2 revision;
		yourself.
	testUser2 login: self class login3.
	response := self post: '/changes' content: (Json render: localRevisions).
	self should: response isOk.
	changes := Json readFrom: response content readStream.
	self should: changes size = 1.
	self should: changes first = testUser2 restUri.! !

!XpfRevisionHandlerTest methodsFor: 'as yet unclassified' stamp: 'mig 4/1/2009 16:07'!
testDeletedChanges

	| response changes localRevisions restUri |
	localRevisions := Dictionary new
		at: testUser restUri put: testUser revision;
		at: testUser2 restUri put: testUser2 revision;
		yourself.
	restUri := testUser2 restUri.
	testUser2 delete.
	response := self post: '/changes' content: (Json render: localRevisions).
	self should: response isOk.
	changes := Json readFrom: response content readStream.
	self should: changes size = 1.
	self should: changes first = restUri.! !

!XpfRevisionHandlerTest methodsFor: 'as yet unclassified' stamp: 'mig 4/1/2009 15:55'!
testNoChanges

	| response changes localRevisions |
	localRevisions := Dictionary new
		at: testUser restUri put: testUser revision;
		at: testUser2 restUri put: testUser2 revision;
		yourself.
	response := self post: '/changes' content: (Json render: localRevisions).
	self should: response isOk.
	changes := Json readFrom: response content readStream.
	self should: changes isEmpty.! !

!Collection methodsFor: '*xpf-server' stamp: 'cs 2/19/2009 14:07'!
add: anObject ifPresent: aBlock
	"Add an object if it is absent. Else run the block"

	(self includes: anObject) 
		ifFalse: [^ self add: anObject]
		ifTrue: [^ aBlock value]! !

TestCase subclass: #XpfTestCase
	instanceVariableNames: 'users projects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

XpfTestCase subclass: #XpfAuthTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfAuthTest methodsFor: 'as yet unclassified' stamp: 'mfp 3/6/2009 11:04'!
testUsuallyAllowed
	| entity user |
	entity := XpfEntity new.
	user := XpfUser new.
	
	self should: (entity isMethod: 'METHOD' allowedFor: user).! !

XpfTestCase subclass: #XpfIterationTest
	instanceVariableNames: 'testProject sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfIterationTest methodsFor: 'running' stamp: 'mt 4/16/2009 11:50'!
setUp

	super setUp.
	testProject := XpfProject addNewProject: (Dictionary newFrom: {'title' -> 'testProject'}).
	sut := XpfIteration new 						
		title: 'Iteration 1'.! !

!XpfIterationTest methodsFor: 'tests' stamp: 'mt 4/16/2009 11:10'!
testAttributes

	self
		checkAccessor: #title on: sut with: 'Iteration 2';
		checkAccessor: #timeIdeal on: sut with: 15.
		
! !

!XpfIterationTest methodsFor: 'tests' stamp: 'mt 4/16/2009 15:30'!
testHasUserStories

	| iterationCount usCount us |
	testProject addIteration: sut.
	iterationCount := testProject iterations size.
	usCount := sut userStories size.
	us := sut addUserStory: XpfUserStory new.
	
	self
		should: sut userStories size = (usCount + 1)
			description: 'User story count for interation is invalid.';
		should: sut userStoriesWithDeleted size = (usCount + 1);
		should: testProject iterations size = iterationCount;
		should: (us iteration = sut).	
	
	us delete.
	self
		should: sut userStories size = usCount;
		should: sut userStoriesWithDeleted size = (usCount + 1);		
		should: testProject iterations size = iterationCount.

	sut delete.! !

!XpfIterationTest methodsFor: 'running' stamp: 'mt 4/16/2009 11:50'!
testMoveUserStory

	| targetIter us |
	testProject addIteration: sut.
	targetIter := testProject addIteration: XpfIteration new.
	us := sut addUserStory: (XpfUserStory new title: 'testMoveUserStory').
	
	self
		should: targetIter userStories isEmpty;
		should: sut userStories first = us.
		
	targetIter addUserStory: us.
	
	self
		should: targetIter userStories first = us;
		should: sut userStories isEmpty.
	! !

!XpfIterationTest methodsFor: 'tests' stamp: 'mt 4/16/2009 11:34'!
testNewAndDelete

	| oldCount us |
	oldCount := testProject iterations size.
	testProject addIteration: sut.
	us := sut addUserStory: XpfUserStory new.
	
	self
		should: testProject iterations size = (oldCount + 1);
		should: (testProject iterations anySatisfy: [:each | each title = sut title]);
		shouldnt: sut isDeleted;
		shouldnt: us isDeleted.

	sut delete.
	self 
		should: testProject iterations size = oldCount;
		should: sut isDeleted;
		should: us isDeleted.! !

!XpfIterationTest methodsFor: 'tests' stamp: 'SR 3/20/2009 21:38'!
testNewFrom

	[XpfIteration newFrom: Dictionary new.]
		on: Error
		do: [self fail].! !

XpfTestCase subclass: #XpfProjectTest
	instanceVariableNames: 'testUser sut testIteration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfProjectTest methodsFor: 'running' stamp: 'mt 4/16/2009 13:22'!
setUp

	super setUp.
	sut := XpfProject new title: 'testProject'.
	testIteration := XpfIteration new title: 'testIteration'.
	testUser := XpfUser new
			login: 'christopher.schuster@student.hpi.uni-potsdam.de';
			userName: 'Christopher Schuster'.
	XpfUser addUser: testUser.! !

!XpfProjectTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:10'!
testAddExistingUserAsDeveloper
	"Add a user as developer to a project. That user should exist."

	| developerCount userCount developer userId |
	XpfUser addUser: testUser.
	userId := testUser id.
	developerCount := sut developers size.
	userCount := XpfUser all size.
	developer := sut attachDeveloper: testUser.

	self
		should: testUser id = userId;
		should: XpfUser all size = userCount;
		should: sut developers size = (developerCount + 1);
		should: (XpfUser all includes: developer);
		should: (sut developers includes: developer).

	sut detachDeveloper: developer.
	self
		should: testUser id = userId;
		should: XpfUser all size = userCount;
		should: sut developers size = developerCount;
		should: (XpfUser all includes: developer);
		shouldnt: (sut developers includes: developer).! !

!XpfProjectTest methodsFor: 'tests' stamp: 'topa 3/3/2009 23:40'!
testAttributes

	self checkAccessor: #title on: sut with: 'testProject'.! !

!XpfProjectTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:09'!
testAuthentication

	self shouldnt: (sut isMethod: 'GET' allowedFor: testUser).

	sut attachDeveloper: testUser.
	self should: (sut isMethod: 'GET' allowedFor: testUser).

	sut detachDeveloper: testUser.
	self shouldnt: (sut isMethod: 'GET' allowedFor: testUser).
	! !

!XpfProjectTest methodsFor: 'tests' stamp: 'mt 4/16/2009 15:38'!
testHasDevelopers

	| developerCount userCount developer |
	developerCount := sut developers size.
	userCount := XpfUser all size.
	developer := sut attachDeveloper: testUser.

	self
		should: testUser id notNil;
		should: XpfUser all size = userCount;
		should: sut developers size = (developerCount + 1);
		should: sut developersWithDeleted size = (developerCount + 1);
		should: (XpfUser all includes: developer);
		should: (sut developers includes: developer).

	sut detachDeveloper: developer.
	self
		should: XpfUser all size = userCount;
		should: sut developers size = developerCount;
		should: sut developersWithDeleted size = developerCount;
		should: (XpfUser all includes: developer);
		shouldnt: (sut developers includes: developer).! !

!XpfProjectTest methodsFor: 'tests' stamp: 'mt 4/16/2009 15:31'!
testHasIterations

	| iterCount iter |
	iterCount := sut iterations size.
	iter := sut addIteration: testIteration.

	self
		should: testIteration id notNil;
		should: sut iterations size = (iterCount + 1);
		should: sut iterationsWithDeleted size = (iterCount + 1).

	iter delete.
	self
		should: sut iterations size = iterCount;
		should: sut iterationsWithDeleted size = (iterCount + 1).! !

!XpfProjectTest methodsFor: 'tests' stamp: 'mt 4/16/2009 11:36'!
testNewAndDelete

	| oldCount |
	oldCount := XpfProject all size.
	XpfProject addProject: sut.
	sut addIteration: testIteration.
		
	self 
		should: XpfProject all size = (oldCount + 1) description: 'Project count is not valid for database.';
		should: (XpfProject all anySatisfy: [ :each | each title = sut title]);
		shouldnt: sut isDeleted;
		shouldnt: testIteration isDeleted.
	
	sut delete.
	self
		should: XpfProject all size = oldCount description: 'Project count is not valid for database after deletion.';
		should: sut isDeleted;
		should: testIteration isDeleted.! !

!XpfProjectTest methodsFor: 'tests' stamp: 'mt 3/16/2009 11:04'!
testNewFrom

	| done |
	[XpfProject newFrom: (Dictionary newFrom: {'title'->'testProject'}).]
		on: Error
		do: [self fail].
	
	[XpfProject newFrom: Dictionary new. done := true]
		on: Error
		do: [done := false].

	self shouldnt: done.! !

XpfTestCase subclass: #XpfRevisionTest
	instanceVariableNames: 'sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfRevisionTest methodsFor: 'running' stamp: 'topa 4/2/2009 18:39'!
setUp

	super setUp.
	sut := XpfProject new title: 'testProject'.! !

!XpfRevisionTest methodsFor: 'running' stamp: 'cs 4/17/2009 12:09'!
testHistorySerialization

	| entity parsedEntity |
	entity := XpfProject new.
	entity historyAddEntry: #changed for: #title from: 'foo' to: 'bar'.
	
	parsedEntity := Json readFrom: (Json render: entity) readStream.
	
	self
		should: ((parsedEntity at: 'col history') isKindOf: Array);
		should: (parsedEntity at: 'col history') size = 2.! !

!XpfRevisionTest methodsFor: 'tests' stamp: 'topa 4/2/2009 18:40'!
testRevisionChange

	| oldRevision |
	oldRevision := sut revision.
	sut title: 'brand new title'.
	self should: sut revision = (oldRevision + 1).! !

XpfTestCase subclass: #XpfTaskTest
	instanceVariableNames: 'testProject testUserStory sut testIteration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfTaskTest methodsFor: 'running' stamp: 'mt 4/16/2009 11:22'!
setUp

	super setUp.
	testProject := XpfProject new title: 'testProject'.
	testIteration := testProject addIteration: (XpfIteration new title: 'testIteration').
	testUserStory := testIteration addNewUserStory: (Dictionary with: 'title' -> 'testUserStory').
	sut := XpfTask new title: 'test task'.
! !

!XpfTaskTest methodsFor: 'tests' stamp: 'mt 4/15/2009 17:16'!
testAttributes

	self
		shouldnt: sut isDone;
		checkAccessor: #title on: sut with: 'test task';
		checkAccessor: #timeEstimated on: sut with: 10;
		checkAccessor: #timeActual on: sut with: 12;
		checkAccessor: #isDone on: sut with: true.! !

!XpfTaskTest methodsFor: 'tests' stamp: 'mt 4/16/2009 15:39'!
testHasDevelopers

	| developer |
	developer := testProject attachDeveloper: (XpfUser addNewUser: (Dictionary with: 'login' -> 'alice')).
	
	self should: sut developers size = 0.
	testUserStory addTask: sut.

	sut attachDeveloper: developer.
	self
		should: sut developers size = 1;
		should: sut developersWithDeleted size = 1;
		should: (developer tasks includes: sut).
	
	sut detachDeveloper: developer.
	self
		should: sut developers size = 0;
		should: sut developersWithDeleted size = 0.

	sut delete.
	developer delete.! !

!XpfTaskTest methodsFor: 'tests' stamp: 'topa 4/3/2009 15:01'!
testIntroducedIsDone

	| oldValue |
	oldValue := sut isDone.
	sut isDone: nil. "this is the case in Old objects"
	
	self should: sut isDone not.	
	sut isDone: oldValue.! !

!XpfTaskTest methodsFor: 'tests' stamp: 'mt 4/15/2009 17:34'!
testNewAndDelete

	| oldCount |
	oldCount := testUserStory tasks size.

	testUserStory addTask: sut.
	self
		should: testUserStory tasks size = (oldCount + 1);
		should: (testUserStory tasks anySatisfy: [:each | each title = 'test task']);
		should: sut userStory = testUserStory;
		shouldnt: sut isDeleted.

	sut delete.
	self
		should: testUserStory tasks size = oldCount;
		should: sut userStory = testUserStory;
		should: sut isDeleted.! !

!XpfTaskTest methodsFor: 'tests' stamp: 'mt 3/16/2009 11:04'!
testNewFrom

	[XpfTask newFrom: Dictionary new.]
		on: Error
		do: [self fail].! !

!XpfTestCase class methodsFor: 'testing' stamp: 'topa 3/3/2009 18:42'!
isAbstract

 	^ self name = #XpfTestCase! !

!XpfTestCase methodsFor: 'test-helper' stamp: 'mt 4/6/2009 17:13'!
checkAccessor: aSelector on: aTargetObject with: anObject
	"test getter/setter aSelector of aTargetObject with anObject"
	
	aTargetObject perform: aSelector asMutator with: anObject.
	self should: (aTargetObject perform: aSelector) = anObject
		! !

!XpfTestCase methodsFor: 'test-helper' stamp: 'topa 2/12/2009 14:03'!
denyAccessor: aSelector on: aTargetObject with: anObject
	"test getter/setter aSelector of aTargetObject with anObject
	 that dont behave like plain accessors.
	"
	self
		shouldnt: (aTargetObject perform: aSelector) == anObject
		! !

!XpfTestCase methodsFor: 'running' stamp: 'mt 3/12/2009 13:05'!
setUp

	super setUp.
	
	XpfProject useTestData: true.
	XpfUser useTestData: true.
	
	XpfProject projects: Set new.
	XpfUser users: Set new.! !

!XpfTestCase methodsFor: 'running' stamp: 'mt 3/12/2009 13:07'!
tearDown
	
	XpfProject useTestData: false.
	XpfUser useTestData: false.
	
	super tearDown.! !

XpfTestCase subclass: #XpfUserStoryTest
	instanceVariableNames: 'testProject sut testIteration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfUserStoryTest methodsFor: 'running' stamp: 'mt 4/16/2009 11:30'!
setUp

	super setUp.
	testProject := XpfProject new title: 'testProject'.
	testIteration := testProject addIteration: (XpfIteration new title: 'testIteration').
	sut := XpfUserStory new 						
		title: 'User Authentication';
		priority: 3;
		timeEstimated: 10.! !

!XpfUserStoryTest methodsFor: 'tests' stamp: 'mt 4/15/2009 17:19'!
testAttributes

	self
		checkAccessor: #title on: sut with: 'US + Tasks verwalten';
		checkAccessor: #priority on: sut with: 5;
		checkAccessor: #timeEstimated on: sut  with: 10;
		checkAccessor: #timeActual on: sut  with: 12. 
		
! !

!XpfUserStoryTest methodsFor: 'tests' stamp: 'mt 4/16/2009 15:33'!
testHasTasks

	| task userStoryCount |
	testIteration addUserStory: sut.
	userStoryCount := testIteration userStories size.
		
	self should: sut tasks size = 0.
	task := sut addTask: XpfTask new. 
	self 
		should: sut tasks size = 1	description: 'Task count for user story is invalid.';
		should: sut tasksWithDeleted size = 1;
		should: testIteration userStories size = userStoryCount;
		should: task userStory = sut;
		shouldnt: task isDeleted.
	
	task delete.
	self
		should: sut tasks size = 0;
		should: sut tasksWithDeleted size = 1;
		should: testIteration userStories size = userStoryCount;
		should: task isDeleted.

	sut delete.! !

!XpfUserStoryTest methodsFor: 'tests' stamp: 'cs 4/17/2009 11:30'!
testHistoryInitial

	self should: sut history size = 5.
	self should: sut history first action = #created.! !

!XpfUserStoryTest methodsFor: 'tests' stamp: 'mt 4/16/2009 12:35'!
testMoveTask

	| targetUS task |
	testProject addIteration: testIteration.
	testIteration addUserStory: sut.
	targetUS := testIteration addUserStory: XpfUserStory new.
	task := sut addTask: (XpfTask new title: 'testMoveTask').
	
	self
		should: targetUS tasks isEmpty;
		should: sut tasks first = task.
		
	targetUS addTask: task.
	
	self
		should: targetUS tasks first = task;
		should: sut tasks isEmpty.! !

!XpfUserStoryTest methodsFor: 'tests' stamp: 'mt 4/16/2009 11:39'!
testNewAndDelete

	| oldCount task |
	oldCount := testIteration userStories size.
	testIteration addUserStory: sut.
	task := sut addTask: XpfTask new.
	
	self
		should: testIteration userStories size = (oldCount + 1);
		should: (testIteration userStories anySatisfy: [:each | each title = sut title]);
		shouldnt: sut isDeleted;
		shouldnt: task isDeleted.

	sut delete.
	self 
		should: testIteration userStories size = oldCount;
		should: sut isDeleted;
		should: task isDeleted.! !

!XpfUserStoryTest methodsFor: 'tests' stamp: 'mt 3/16/2009 11:05'!
testNewFrom

	[XpfUserStory newFrom: Dictionary new.]
		on: Error
		do: [self fail].! !

XpfTestCase subclass: #XpfUserTest
	instanceVariableNames: 'sut testProject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Tests'!

!XpfUserTest class methodsFor: 'fixture-data' stamp: 'topa 3/3/2009 22:59'!
login

	^ 'test.user'! !

!XpfUserTest class methodsFor: 'fixture-data' stamp: 'topa 3/3/2009 22:59'!
password

	^ 'TEST'! !

!XpfUserTest methodsFor: 'running' stamp: 'mt 4/16/2009 13:20'!
setUp

	super setUp.
	sut := XpfUser new
				login: self class login;
				password: self class password.
	testProject :=  XpfProject new title: 'xpforums'.
	XpfProject addProject: testProject.! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 4/16/2009 13:18'!
testAddExistingProjectToUser
	"Add a project to the user. That project should exist."

	| projectCount globalProjectCount projectId |
	XpfProject addProject: testProject.
	projectId := testProject id.
	projectCount := sut projects size.
	globalProjectCount := XpfProject all size.
	
	sut attachProject: testProject.
	self
		should: testProject id = projectId;
		should: XpfProject all size = globalProjectCount;
		should: sut projects size = (projectCount + 1);
		should: (XpfProject all includes: testProject);
		should: (sut projects includes: testProject).
	
	sut detachProject: testProject.
	self
		should: testProject id = projectId;
		should: XpfProject all size = globalProjectCount;
		should: sut projects size = projectCount;
		should: (XpfProject all includes: testProject);
		shouldnt: (sut projects includes: testProject).
		! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 3/4/2009 14:02'!
testAttributes

	self
		checkAccessor: #login on: sut with: 'marcel.taeumel';
		checkAccessor: #userName on: sut with: 'Marcel Taeumel';
		checkAccessor: #email on: sut with: 'marcel.taeumel@student.hpi.uni-potsdam.de'.! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 4/16/2009 15:42'!
testHasProjects
	| projectCount globalProjectCount |
	projectCount := sut projects size.
	globalProjectCount := XpfProject all size.
	
	sut attachProject: testProject.
	self
		should: testProject id notNil;
		should: XpfProject all size = globalProjectCount;
		should: sut projects size = (projectCount + 1);
		should: sut projectsWithDeleted size = (projectCount + 1);
		should: (XpfProject all includes: testProject);
		should: (sut projects includes: testProject).
	
	sut detachProject: testProject.
	self
		should: XpfProject all size = globalProjectCount;
		should: sut projects size = projectCount;
		should: sut projectsWithDeleted size = projectCount;		
		should: (XpfProject all includes: testProject);
		shouldnt: (sut projects includes: testProject).
		! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 4/6/2009 15:04'!
testNewAndDelete

	| oldCount |	
	oldCount := XpfUser all size.	
	self shouldnt: (XpfUser all anySatisfy: [ :each | each login = sut login]).

	XpfUser addUser: sut.
	self 
		should: XpfUser all size = (oldCount + 1);
		should: (XpfUser all anySatisfy: [ :each | each login = sut login]);
		shouldnt: sut isDeleted.
	
	sut delete.
	self
		should: XpfUser all size = oldCount;
		should: sut isDeleted.! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 3/16/2009 11:04'!
testNewFrom

	| done |
	[XpfUser newFrom: (Dictionary newFrom: {'login'->'test@example.com'}).]
		on: Error
		do: [self fail].
	
	[XpfUser newFrom: Dictionary new. done := true]
		on: Error
		do: [done := false].

	self shouldnt: done.! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 3/6/2009 14:05'!
testPasswordHashing

	self
		"shouldnt: (sut respondsTo: #password);"
		should: sut password = nil;
		should: (sut authenticateBy: self class password);
		shouldnt: (sut authenticateBy: self class password , 'NOT').
		! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 4/6/2009 15:04'!
testUserAuthenticationFail

	XpfUser addUser: sut.
	self
		should: (XpfUser authenticate: self class login             by: self class password , 'NOT' ) isNil;
		should: (XpfUser authenticate: self class login , 'NOT' by: self class password             ) isNil;
		should: (XpfUser authenticate: self class login , 'NOT' by: self class password , 'NOT') isNil.
	sut delete.! !

!XpfUserTest methodsFor: 'tests' stamp: 'mt 4/6/2009 15:05'!
testUserAuthenticationSuccess

	XpfUser addUser: sut.
	self should: sut = (XpfUser authenticate: self class login by: self class password).
	sut delete.! !
XpfProject initialize!
XpfUser initialize!
